<?php

	App::uses('VebraAppController', 'Vebra.Controller');
	App::uses('Branch', 'Vebra.Model');

	/*
	 * To change this license header, choose License Headers in Project Properties.
	 * To change this template file, choose Tools | Templates
	 * and open the template in the editor.
	 */

	/**
	 * Description of PropertiesController
	 *
	 * @author Luke
	 */
	class PropertiesController extends VebraAppController {

		const FILTERS_FORM_NAME = 'property_filters';

		public $components = array('EvContactForm.InjectContactForm');
		public $helpers = array('EvContactForm.ShowContactForm');
		public $paginate = array(
			'limit' => 10,
			'order' => 'Property.price DESC',
			'contain' => array(
				'Paragraph',
				'Image'
			)
		);
		protected $allowedFilters = array(
			'area', // this needs to be in the URL, but we ignore it in the switch statement below
			'latitude',
			'longitude',
			'type',
			'min_price',
			'max_price',
			'min_bedrooms',
			'max_bedrooms'
		);

		public function beforeFilter() {
			parent::beforeFilter();
			$this->Auth->allow();
			return $this;
		}

		public function index() {
			// Ge tavailable property types
			$property_types = $this->Property->find('all', array(
				'fields' => array(
					'DISTINCT (Property.type) AS property_type',
				),
				'order' => array(
					'Property.type ASC'
				)
			));

			$property_options = array();
			foreach ($property_types as $type) {
				$property_options[$type['Property']['property_type']] = $type['Property']['property_type'];
			}

			$this->set('property_types', $property_options);
			$this->set('min_prices', $this->Property->getMinPrices());
			$this->set('max_prices', $this->Property->getMaxPrices());
			$this->set('min_bedrooms', $this->Property->getMinBedrooms());
			$this->set('max_bedrooms', $this->Property->getMaxBedrooms());

			// POST-REDIRECT-GET (for named params in CakePHP)
			if (($this->request->is('post') || $this->request->is('put')) && isset($this->request->data[static::FILTERS_FORM_NAME])) {
				$filter_url['controller'] = $this->request->params['controller'];
				$filter_url['action'] = $this->request->params['action'];
				$filter_url['page'] = 1;

				// Unset anything we weren't expecting. Yeah, this is clever, you know it (Y)
				foreach (array_diff(array_keys($this->request->data[static::FILTERS_FORM_NAME]), $this->_getAllowedFilters()) as $key) {
					unset($this->request->data[static::FILTERS_FORM_NAME][$key]);
				}

				// for each filter we will add a GET parameter for the generated url
				foreach ($this->request->data[static::FILTERS_FORM_NAME] as $name => $value) {
					if ($value) {
						if ($name === 'latitude' || $name === 'longitude') { // we can't make a slug out of these, or it's not gonna work. We really need to do this based on the param type, but whatever
							$filter_url[$name] = urlencode($value);
						} else {
							$filter_url[$name] = InflectorExt::slug($value);
						}
					}
				}

				return $this->redirect($filter_url);
			} else {
				$conditions = array();
				$has_min_price = $has_max_price = $has_min_bedrooms = $has_max_bedrooms = false;

				$conditions['Property.solddate'] = null; // we only ever want the unsold properties
				$distance_query = ''
						. '('
						. '('
						. 'ACOS('
						. 'SIN(@latitude * PI() / 180) '
						. '* SIN(`Property`.latitude * PI() / 180) '
						. '+ COS(@latitude * PI() / 180) '
						. '* COS(`Property`.latitude * PI() / 180) '
						. '* COS((@longitude - `Property`.longitude) * PI() / 180)'
						. ') '
						. '* 180 / PI()'
						. ') '
						. '* 60 '
						. '* 1.1515'
						. ')'; // this looks scary (and unless you're awesome at maths, it's meant to), but it's just trying to figure out the distanec between two lat/long pairs
				foreach ($this->params['named'] as $param_name => $value) {
					if (!in_array($param_name, array('page', 'sort', 'direction', 'limit'))) { // Don't apply the default named parameters used for pagination
						switch ($param_name) {
							case 'latitude':
								$distance_query = str_replace('@latitude', $value, $distance_query);
								break;

							case 'longitude':
								$distance_query = str_replace('@longitude', $value, $distance_query);
								break;

							case 'min_price':
								$has_min_price = true;
								break;

							case 'max_price':
								$has_max_price = true;
								break;

							case 'min_bedrooms':
								$has_min_bedrooms = true;
								break;

							case 'max_bedrooms':
								$has_max_bedrooms = true;
								break;

							case 'area':
								// do nothing with it, just make sure we don't try to query it, as it's pretty much there to look nice and pre-populate the forms
								break;

							default: // by default we do a simple col = val
								$conditions[$param_name] = $value;
								break;
						}

						// We need to do a separate switch, because the overwhelming majority of values can remain as they are in the URL when being used in the form
						switch ($param_name) {
							case 'area':
								$this->request->data[static::FILTERS_FORM_NAME][$param_name] = InflectorExt::humanizeSlug($value, ', ');
								break;
							default:
								$this->request->data[static::FILTERS_FORM_NAME][$param_name] = $value;
								break;
						}
					}
				}

				// Build distance conditions based on lat / long yielded from autocomplete area search
				if (isset($this->params['named']['latitude']) && isset($this->params['named']['latitude'])) {
					$radius = 1; // miles
					$conditions[$distance_query . ' <'] = $radius;
				}

				// Build price conditions
				if ($has_min_price && $has_max_price) {
					$conditions['Property.price BETWEEN ? AND ?'] = array($this->params['named']['min_price'], $this->params['named']['max_price']);
				} else if ($has_min_price) {
					$conditions['Property.price >='] = $this->params['named']['min_price'];
				} else if ($has_max_price) {
					$conditions['Property.price <='] = $this->params['named']['max_price'];
				}

				// Build bedroom conditions
				if ($has_min_bedrooms && $has_max_bedrooms) {
					$conditions['Property.bedrooms BETWEEN ? AND ?'] = array($this->params['named']['min_bedrooms'], $this->params['named']['max_bedrooms']);
				} else if ($has_min_bedrooms) {
					$conditions['Property.bedrooms >='] = $this->params['named']['min_bedrooms'];
				} else if ($has_max_bedrooms) {
					$conditions['Property.bedrooms <='] = $this->params['named']['max_bedrooms'];
				}

				// Get all properties (including filters etc.)
				$properties = $this->paginate('Property', $conditions, array('price', 'uploaded'));
				$this->set('properties', $properties);
				$this->set('title_for_layout', 'All Properties | ' . Configure::read('SiteSetting.site_title'));
			}
		}

		public function view($id) {
			$params = array(
				'conditions' => array(
					'Property.id' => $id
				),
				'contain' => array(
					'Paragraph',
					'Image',
					'FloorplanImage',
					'DetailmapImage',
					'StreetmapImage',
					'TownmapImage',
					'EpcImage',
					'BlurredImage',
					'Branch',
				)
			);

			$property = $this->Property->find('first', $params);
			$this->set('property', $property);

			$branch_name = $property['Branch']['name'];
			App::uses('SiteSettingCategory', 'SiteSettings.Model');
			$site_setting_category = (new SiteSettingCategory())->find('first', array('conditions' => array('name' => $branch_name)));
			if (is_array($site_setting_category) && !empty($site_setting_category)) {
				$this->set('telephone', Configure::read('SiteSetting.' . strtolower($branch_name) . '_telephone'));
			}

			$this->set('available_viewing_slots', explode("\n", Configure::read('SiteSetting.available_viewing_slots')));
			$this->InjectContactForm->display(EV_CONTACT_FORM_ARRANGE_A_VIEWING, 'arrange_viewing');

			if ($this->request->is('ajax')) {
				$this->set('name', 'arrange_viewing');
				$this->render('EvContactForm.Common/ajax-default', 'ajax');
			}

			$this->set('title_for_layout', implode(', ', array($property['Property']['street'], $property['Property']['town'], $property['Property']['county'])) . ' | ' . Configure::read('SiteSetting.site_title'));
		}

		protected function _adminFormFields() {
			return array(); // we don't want any of the property information to be user-editable because it'll just get overwritten by the API, anyway
		}

		protected function _getAllowedFilters() {
			return $this->allowedFilters;
		}

	}
